@@ -16,10 +16,10 @@
 ###############################################################################
 
 from copy import deepcopy
+from inspect import getmembers
 
 
 __all__ = [
-    'super_vars',
     'BoolConfigOption',
     'ConfigOption',
     'ConfigSection',
@@ -36,15 +36,10 @@ NO_DEFAULT = object()
 _internal = object.__dict__.keys() + ['__module__']
 
 
-def super_vars(obj):
-    """An extended version of vars() that walks all base classes."""
-    items = {}
-    if hasattr(obj, '__mro__'):
-        bases = map(vars, obj.__mro__)
-        map(items.update, bases)
-    else:
-        items = vars(obj)
-    return items
+def get_config_objects(obj):
+    objects = ((n, o) for (n, o) in getmembers(obj)
+        if isinstance(o, (ConfigSection, ConfigOption)))
+    return objects
 
 
 class Schema(object):
@@ -63,56 +58,53 @@ class Schema(object):
     One ConfigOption comes already defined in Schema, 'includes' in the
     '__main__' section, that allows configuration files to include other
     configuration files.
+
     """
 
-    def __new__(cls):
-        instance = super(Schema, cls).__new__(cls)
-
-        # override class attributes with instance attributes to correctly
-        # handle schema inheritance
-        schema_attributes = filter(
-            lambda x: x not in _internal and
-                isinstance(getattr(cls, x), (ConfigSection, ConfigOption)),
-            super_vars(cls))
-        for attr in schema_attributes:
-            setattr(instance, attr, deepcopy(getattr(cls, attr)))
-
-        return instance
-
     def __init__(self):
         self.includes = LinesConfigOption(item=StringConfigOption())
         self._sections = {}
-        defaultSection = None
-        for attname in super_vars(self.__class__):
-            att = getattr(self, attname)
-            if isinstance(att, ConfigSection):
-                att.name = attname
-                self._sections[attname] = att
-                for optname in super_vars(att):
-                    opt = getattr(att, optname)
-                    if isinstance(opt, ConfigOption):
-                        opt.name = optname
-                        opt.section = att
-            elif isinstance(att, ConfigOption):
-                if defaultSection is None:
-                    defaultSection = ConfigSection()
-                    defaultSection.name = '__main__'
-                    self._sections['__main__'] = defaultSection
-                att.name = attname
-                att.section = defaultSection
-                setattr(defaultSection, attname, att)
+        # add section and options to the schema
+        for name, item in get_config_objects(self.__class__):
+            self._add_item(name, item)
+
+    def _add_item(self, name, item):
+        """Add a top-level item to the schema."""
+        item.name = name
+        if isinstance(item, ConfigSection):
+            self._add_section(name, item)
+        elif isinstance(item, ConfigOption):
+            self._add_option(name, item)
+        # override class attributes with instance attributes to correctly
+        # handle schema inheritance
+        setattr(self, name, deepcopy(item))
+
+    def _add_section(self, name, section):
+        """Add a top-level section to the schema."""
+        self._sections[name] = section
+        for opt_name, opt in get_config_objects(section):
+            opt.name = opt_name
+            opt.section = section
+
+    def _add_option(self, name, option):
+        """Add a top-level option to the schema."""
+        section = self._sections.setdefault('__main__', ConfigSection(name='__main__'))
+        option.section = section
+        setattr(section, name, option)
 
     def __eq__(self, other):
         return (self._sections == other._sections and
                 self.includes == other.includes)
 
     def is_valid(self):
+        """Return whether the schema has a valid structure."""
         explicit_default_section = isinstance(getattr(self, '__main__', None),
                                               ConfigSection)
         is_valid = not explicit_default_section
         return is_valid
 
     def has_section(self, name):
+        """Return whether the schema as a given section."""
         """Return True if a ConfigSection with the given name is available"""
         return name in self._sections.keys()
 
@@ -132,6 +124,7 @@ class Schema(object):
         If section is omitted, returns all the options in the configuration
         file, flattening out any sections.
         To get options from the default section, specify section='__main__'
+
         """
         if isinstance(section, basestring):
             section = self.section(section)
@@ -140,7 +133,8 @@ class Schema(object):
             for s in self.sections():
                 options += self.options(s)
         elif section.name == '__main__':
-            options = [getattr(self, att) for att in super_vars(self.__class__) 
+            class_config_objects = get_config_objects(self.__class__)
+            options = [getattr(self, att) for att, _ in class_config_objects
                            if isinstance(getattr(self, att), ConfigOption)]
         else:
             options = section.options()
@@ -154,6 +148,7 @@ class ConfigSection(object):
 
     After instantiating the Schema, each ConfigSection will know its own
     name.
+
     """
     def __init__(self, name=''):
         self.name = name
@@ -212,6 +207,7 @@ class ConfigOption(object):
 
     In runtime, after instantiating the Schema, each ConfigOption will also
     know its own name and to which section it belongs.
+
     """
 
     require_parser = False
@@ -262,6 +258,7 @@ class ConfigOption(object):
         return None
 
     def parse(self, value):
+        """Parse the given value."""
         raise NotImplementedError()
 
 
@@ -272,6 +269,11 @@ class BoolConfigOption(ConfigOption):
         return False
 
     def parse(self, value, raw=False):
+        """Parse the given value.
+
+        If *raw* is *True*, return the value unparsed.
+
+        """
         if raw:
             return value
 
@@ -290,6 +292,11 @@ class IntConfigOption(ConfigOption):
         return 0
 
     def parse(self, value, raw=False):
+        """Parse the given value.
+
+        If *raw* is *True*, return the value unparsed.
+
+        """
         if raw:
             return value
 
@@ -309,12 +316,28 @@ class LinesConfigOption(ConfigOption):
     if remove_duplicates == True, duplicate elements in the lines will be
     removed.  Only the first occurrence of any item will be kept,
     otherwise the general order of the list will be preserved.
+
     """
 
+    def __init__(self, name='', item=None, raw=False, default=NO_DEFAULT,
+        fatal=False, help='', action='store', remove_duplicates=False):
+        super(LinesConfigOption, self).__init__(name=name, raw=raw,
+            default=default, fatal=fatal, help=help, action=action)
+        self.item = item
+        self.require_parser = item.require_parser
+        self.raw = item.raw
+        self.remove_duplicates = remove_duplicates
+
     def _get_default(self):
         return []
 
     def parse(self, value, parser=None, raw=False):
+        """Parse the given value.
+
+        A *parser* object is used to parse individual list items.
+        If *raw* is *True*, return the value unparsed.
+
+        """
         def _parse_item(value):
             if self.require_parser:
                 value = self.item.parse(value, parser=parser, raw=raw)
@@ -330,37 +353,39 @@ class LinesConfigOption(ConfigOption):
             items = filtered_items
         return items
 
-    def __init__(self, item, raw=False, default=NO_DEFAULT, fatal=False,
-        help='', action='store', remove_duplicates=False):
-        super(LinesConfigOption, self).__init__(raw=raw, default=default,
-            fatal=fatal, help=help, action=action)
-        self.item = item
-        self.require_parser = item.require_parser
-        self.raw = item.raw
-        self.remove_duplicates = remove_duplicates
-
 
 class StringConfigOption(ConfigOption):
     """A ConfigOption that is parsed into a string.
 
     If null==True, a value of 'None' will be parsed in to None instead of
     just leaving it as the string 'None'.
+
     """
 
+    def __init__(self, name='', raw=False, default=NO_DEFAULT, fatal=False,
+        null=False, help='', action='store'):
+        self.null = null
+        super(StringConfigOption, self).__init__(name=name, raw=raw,
+            default=default, fatal=fatal, help=help, action=action)
+
     def _get_default(self):
         return '' if not self.null else None
 
     def parse(self, value, raw=False):
+        """Parse the given value.
+
+        If *raw* is *True*, return the value unparsed.
+
+        """
         if raw:
-            return value
-
-        return unicode(value)
-
-    def __init__(self, raw=False, default=NO_DEFAULT, fatal=False, null=False,
-                 help='', action='store'):
-        self.null = null
-        super(StringConfigOption, self).__init__(raw=raw, default=default,
-            fatal=fatal, help=help, action=action)
+            result = value
+        elif self.null:
+            result = None if value in (None, 'None') else value
+        elif isinstance(value, basestring):
+            result = value
+        else:
+            result = repr(value)
+        return result
 
 
 class TupleConfigOption(ConfigOption):
@@ -368,18 +393,24 @@ class TupleConfigOption(ConfigOption):
 
     The number of items in the tuple should be specified with the 'length'
     constructor argument.
+
     """
 
-    def __init__(self, length=0, raw=False, default=NO_DEFAULT, fatal=False,
-                 help='', action='store'):
-        super(TupleConfigOption, self).__init__(raw=raw, default=default,
-            fatal=fatal, help=help, action=action)
+    def __init__(self, name='', length=0, raw=False, default=NO_DEFAULT,
+        fatal=False, help='', action='store'):
+        super(TupleConfigOption, self).__init__(name=name, raw=raw,
+            default=default, fatal=fatal, help=help, action=action)
         self.length = length
 
     def _get_default(self):
         return ()
 
     def parse(self, value, raw=False):
+        """Parse the given value.
+
+        If *raw* is *True*, return the value unparsed.
+
+        """
         parts = [part.strip() for part in value.split(',')]
         if parts == ['()']:
             result = ()
@@ -405,10 +436,12 @@ class DictConfigOption(ConfigOption):
     argument, that should be in turn a dictionary.  spec's keys are the
     available keys for the config file, and spec's values should be
     ConfigOptions that will be used to parse the values in the config file.
+
     """
+
     require_parser = True
 
-    def __init__(self, spec=None, strict=False, raw=False,
+    def __init__(self, name='', spec=None, strict=False, raw=False,
                  default=NO_DEFAULT, fatal=False, help='', action='store',
                  item=None):
         if spec is None:
@@ -418,8 +451,8 @@ class DictConfigOption(ConfigOption):
         self.spec = spec
         self.strict = strict
         self.item = item
-        super(DictConfigOption, self).__init__(raw=raw, default=default,
-            fatal=fatal, help=help, action=action)
+        super(DictConfigOption, self).__init__(name=name, raw=raw,
+            default=default, fatal=fatal, help=help, action=action)
 
     def _get_default(self):
         default = {}
@@ -428,6 +461,12 @@ class DictConfigOption(ConfigOption):
         return default
 
     def parse(self, section, parser=None, raw=False):
+        """Parse the given value.
+
+        A *parser* object is used to parse individual dict items.
+        If *raw* is *True*, return the value unparsed.
+
+        """
         parsed = dict(parser.items(section))
         result = {}
 
