@@ -20,10 +20,19 @@ import os
 import shutil
 import tempfile
 import unittest
+from ConfigParser import (
+    DEFAULTSECT,
+    InterpolationDepthError,
+    InterpolationMissingOptionError,
+    InterpolationSyntaxError,
+    NoSectionError,
+)
 from StringIO import StringIO
 
-from ConfigParser import (InterpolationMissingOptionError,
-    InterpolationDepthError, NoSectionError)
+from mock import (
+    Mock,
+    patch_object,
+)
 
 from configglue.pyschema.parser import (
     CONFIG_FILE_ENCODING,
@@ -211,7 +220,7 @@ class TestInterpolation(unittest.TestCase):
         rawval = '%(bar)'
         vars = {'foo': '%(bar)s', 'bar': 'pepe'}
         parser = SchemaConfigParser(MySchema())
-        self.assertRaises(ValueError, parser._interpolate, section, option,
+        self.assertRaises(InterpolationSyntaxError, parser._interpolate, section, option,
                           rawval, vars)
 
     def test_interpolate_across_sections(self):
@@ -236,7 +245,7 @@ class TestInterpolation(unittest.TestCase):
             baz = ConfigSection()
             baz.wham = IntConfigOption()
 
-        config = StringIO("[foo]\nbar=%(wham)\n[baz]\nwham=42")
+        config = StringIO("[foo]\nbar=%(wham)s\n[baz]\nwham=42")
         parser = SchemaConfigParser(MySchema())
         parser.readfp(config)
         self.assertRaises(InterpolationMissingOptionError, parser.get,
@@ -311,7 +320,7 @@ class TestInterpolation(unittest.TestCase):
 
     def test_get_interpolation_keys_dict(self):
         class MySchema(Schema):
-            foo = DictConfigOption({'a': IntConfigOption()})
+            foo = DictConfigOption(spec={'a': IntConfigOption()})
         config = StringIO("[__noschema__]\nbar=4\n[__main__]\nfoo=mydict\n[mydict]\na=%(bar)s")
         expected = ('mydict', set([]))
 
@@ -342,6 +351,22 @@ class TestInterpolation(unittest.TestCase):
         value = parser._interpolate_value('__main__', 'foo')
         self.assertEqual(value, expected_value)
 
+    def test_interpolate_value_no_keys(self):
+        class MySchema(Schema):
+            foo = TupleConfigOption(2)
+        config = StringIO("[__main__]\nfoo=%(bar)s,%(bar)s")
+
+        mock_get_interpolation_keys = Mock(return_value=('%(bar)s', None))
+
+        parser = SchemaConfigParser(MySchema())
+        parser.readfp(config)
+        with patch_object(parser, '_get_interpolation_keys',
+                mock_get_interpolation_keys):
+
+            value = parser._interpolate_value('__main__', 'foo')
+            self.assertEqual(value, None)
+
+
     def test_get_with_raw_value(self):
         class MySchema(Schema):
             foo = StringConfigOption(raw=True)
@@ -355,7 +380,7 @@ class TestInterpolation(unittest.TestCase):
 
     def test_interpolate_parse_dict(self):
         class MySchema(Schema):
-            foo = DictConfigOption({'a': IntConfigOption()})
+            foo = DictConfigOption(spec={'a': IntConfigOption()})
         config = StringIO("[__noschema__]\nbar=4\n[__main__]\nfoo=mydict\n[mydict]\na=%(bar)s")
         expected = {'__main__': {'foo': {'a': 4}}}
 
@@ -525,7 +550,7 @@ class TestSchemaConfigParser(unittest.TestCase):
 
     def test_extra_sections(self):
         class MySchema(Schema):
-            foo = DictConfigOption({'bar': IntConfigOption()})
+            foo = DictConfigOption(spec={'bar': IntConfigOption()})
 
         config = StringIO("[__main__]\nfoo=mydict\n[mydict]\nbar=1")
         parser = SchemaConfigParser(MySchema())
@@ -539,7 +564,7 @@ class TestSchemaConfigParser(unittest.TestCase):
     def test_multiple_extra_sections(self):
         class MySchema(Schema):
             foo = LinesConfigOption(
-                item=DictConfigOption({'bar': IntConfigOption()}))
+                item=DictConfigOption(spec={'bar': IntConfigOption()}))
 
         config = StringIO('[__main__]\nfoo=d1\n    d2\n    d3\n'
                           '[d1]\nbar=1\n[d2]\nbar=2\n[d3]\nbar=3')
@@ -589,9 +614,10 @@ class TestSchemaConfigParser(unittest.TestCase):
 
     def test_multi_file_dict_config(self):
         class MySchema(Schema):
-            foo = DictConfigOption({'bar': IntConfigOption(),
-                                    'baz': IntConfigOption()},
-                                   strict=True)
+            foo = DictConfigOption(spec={
+                'bar': IntConfigOption(),
+                'baz': IntConfigOption(),
+            }, strict=True)
         config1 = StringIO('[__main__]\nfoo=mydict\n[mydict]\nbar=1\nbaz=1')
         config2 = StringIO('[mydict]\nbaz=2')
         expected_values = {'__main__': {'foo': {'bar': 1, 'baz': 2}}}
@@ -604,9 +630,10 @@ class TestSchemaConfigParser(unittest.TestCase):
     def test_multi_file_dict_list_config(self):
         class MySchema(Schema):
             foo = LinesConfigOption(
-                item=DictConfigOption({'bar': IntConfigOption(),
-                                       'baz': IntConfigOption()},
-                                      strict=True))
+                item=DictConfigOption(spec={
+                    'bar': IntConfigOption(),
+                    'baz': IntConfigOption(),
+                }, strict=True))
 
         config1 = StringIO('[__main__]\nfoo=mydict\n[mydict]\nbar=1\nbaz=1')
         expected_values = {'__main__': {'foo': [{'bar': 1, 'baz': 1}]}}
@@ -692,21 +719,50 @@ class TestSchemaConfigParser(unittest.TestCase):
             self.assertEqual(self.parser._dirty,
                 {f.name: {'__main__': {'foo': '2'}}})
 
+    def test_write(self):
+        class MySchema(Schema):
+            foo = StringConfigOption()
+            DEFAULTSECT = ConfigSection()
+        parser = SchemaConfigParser(MySchema())
+        expected = u"[{0}]\nbaz = 2\n\n[__main__]\nfoo = bar".format(DEFAULTSECT)
+        config = StringIO(expected)
+        parser.readfp(config)
+
+        # create config file
+        fp, filename = tempfile.mkstemp()
+        try:
+            parser.write(open(filename, 'w'))
+            result = open(filename, 'r').read().strip()
+            self.assertEqual(result, expected)
+        finally:
+            # remove the file
+            os.unlink(filename)
+
     def test_save_config(self):
-        expected_output = u'[__main__]\nfoo = 42\n\n'
-        config = StringIO(u'[__main__]\nfoo=42')
+        expected = u'[__main__]\nfoo = 42'
+        self._check_save_file(expected)
+
+    def test_save_config_non_ascii(self):
+        expected = u'[__main__]\nfoo = fóobâr'
+        self._check_save_file(expected)
+
+    def _check_save_file(self, expected):
+        config = StringIO(expected.encode(CONFIG_FILE_ENCODING))
         self.parser.readfp(config)
 
         # create config file
         fp, filename = tempfile.mkstemp()
-        self.parser.save(open(filename, 'w'))
-        self.assertEqual(open(filename, 'r').read(), expected_output)
+        try:
+            self.parser.save(open(filename, 'w'))
+            result = open(filename, 'r').read().strip()
+            self.assertEqual(result.decode(CONFIG_FILE_ENCODING), expected)
 
-        self.parser.save(filename)
-        self.assertEqual(open(filename, 'r').read(), expected_output)
-
-        # remove the file
-        os.unlink(filename)
+            self.parser.save(filename)
+            result = open(filename, 'r').read().strip()
+            self.assertEqual(result.decode(CONFIG_FILE_ENCODING), expected)
+        finally:
+            # remove the file
+            os.unlink(filename)
 
     def test_save_config_same_files(self):
         def setup_config():
@@ -825,6 +881,15 @@ class TestParserIsValid(unittest.TestCase):
 
         self.assertFalse(self.parser.is_valid())
 
+    def test_parse_invalid_section_with_report(self):
+        config = StringIO("[bar]\nbaz=foo")
+        self.parser.readfp(config)
+
+        valid, errors = self.parser.is_valid(report=True)
+        self.assertFalse(valid)
+        self.assertEqual(errors,
+            [u'Sections in configuration do not match schema: bar'])
+
     def test_different_sections(self):
         config = StringIO("[__main__]\nfoo=1\n[bar]\nbaz=2")
         self.parser.readfp(config)
@@ -855,7 +920,7 @@ class TestParserIsValid(unittest.TestCase):
 
     def test_extra_sections(self):
         class MySchema(Schema):
-            foo = DictConfigOption({'bar': IntConfigOption()})
+            foo = DictConfigOption(spec={'bar': IntConfigOption()})
 
         config = StringIO("[__main__]\nfoo=mydict\n[mydict]\nbar=1")
         parser = SchemaConfigParser(MySchema())
@@ -886,7 +951,7 @@ baz=42
 
     def test_extra_sections_with_nested_dicts_strict(self):
         class MySchema(Schema):
-            foo = DictConfigOption({'bar': DictConfigOption()}, strict=True)
+            foo = DictConfigOption(spec={'bar': DictConfigOption()}, strict=True)
 
         config = StringIO("""
 [__main__]
@@ -996,7 +1061,7 @@ swoosh = 4
     def test_multiple_extra_sections(self):
         class MySchema(Schema):
             foo = LinesConfigOption(
-                item=DictConfigOption({'bar': IntConfigOption()}))
+                item=DictConfigOption(spec={'bar': IntConfigOption()}))
 
         config = StringIO('[__main__]\nfoo=d1\n    d2\n    d3\n'
                           '[d1]\nbar=1\n[d2]\nbar=2\n[d3]\nbar=3')
