@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 ###############################################################################
 # 
 # configglue -- glue for your apps' configuration
@@ -16,6 +17,7 @@
 ###############################################################################
 
 import unittest
+from copy import deepcopy
 from StringIO import StringIO
 
 from configglue.pyschema.parser import SchemaConfigParser
@@ -153,46 +155,76 @@ class TestSchemaInheritance(unittest.TestCase):
         # test on the other schema
         self.assertFalse(hasattr(self.other.foo, 'baz'))
 
+    def test_merge_inherited(self):
+        class SchemaA(Schema):
+            foo = ConfigSection()
+            foo.bar = IntConfigOption()
+            bar = IntConfigOption()
+
+        class SchemaB(SchemaA):
+            foo = deepcopy(SchemaA.foo)
+            foo.baz = IntConfigOption()
+
+        # SchemaB inherits attributes from SchemaA and merges its own
+        # attributes into
+        schema = SchemaB()
+        section_names = set(s.name for s in schema.sections())
+        option_names = set(o.name for o in schema.options('__main__'))
+        foo_option_names = set(o.name for o in schema.options('foo'))
+        self.assertEqual(section_names, set(['__main__', 'foo']))
+        self.assertEqual(option_names, set(['bar']))
+        self.assertEqual(foo_option_names, set(['bar', 'baz']))
+
+        # SchemaB inheritance does not affect SchemaA
+        schema = SchemaA()
+        section_names = set(s.name for s in schema.sections())
+        option_names = set(o.name for o in schema.options('__main__'))
+        foo_option_names = set(o.name for o in schema.options('foo'))
+        self.assertEqual(section_names, set(['__main__', 'foo']))
+        self.assertEqual(option_names, set(['bar']))
+        self.assertEqual(foo_option_names, set(['bar']))
+
 
 class TestStringConfigOption(unittest.TestCase):
+    def setUp(self):
+        self.opt = StringConfigOption()
+
     def test_init_no_args(self):
-        opt = StringConfigOption()
-        self.assertFalse(opt.null)
+        self.assertFalse(self.opt.null)
 
     def test_init_null(self):
         opt = StringConfigOption(null=True)
         self.assertTrue(opt.null)
 
-    def test_parse_string(self):
-        class MySchema(Schema):
-            foo = StringConfigOption(null=True)
-        config = StringIO("[__main__]\nfoo = 42")
-        expected_values = {'__main__': {'foo': '42'}}
-        schema = MySchema()
-        parser = SchemaConfigParser(schema)
-        parser.readfp(config)
-        self.assertEqual(parser.values(), expected_values)
+    def test_parse_ascii_string(self):
+        value = self.opt.parse('42')
+        self.assertEqual(value, '42')
 
-        config = StringIO("[__main__]\nfoo = ")
-        expected_values = {'__main__': {'foo': ''}}
-        parser = SchemaConfigParser(schema)
-        parser.readfp(config)
-        self.assertEqual(parser.values(), expected_values)
+    def test_parse_empty_string(self):
+        value = self.opt.parse('')
+        self.assertEqual(value, '')
 
-        config = StringIO("[__main__]\nfoo = None")
-        expected_values = {'__main__': {'foo': None}}
-        parser = SchemaConfigParser(schema)
-        parser.readfp(config)
-        self.assertEqual(parser.values(), expected_values)
+    def test_parse_null_string(self):
+        opt = StringConfigOption(null=True)
+        value = opt.parse(None)
+        self.assertEqual(value, None)
 
-        class MySchema(Schema):
-            foo = StringConfigOption()
-        config = StringIO("[__main__]\nfoo = None")
-        expected_values = {'__main__': {'foo': 'None'}}
-        schema = MySchema()
-        parser = SchemaConfigParser(schema)
-        parser.readfp(config)
-        self.assertEqual(parser.values(), expected_values)
+    def test_None_string(self):
+        value = self.opt.parse('None')
+        self.assertEqual(value, 'None')
+
+    def test_parse_nonascii_string(self):
+        foo = StringConfigOption()
+        value = foo.parse('foóbâr')
+        self.assertEqual(value, 'foóbâr')
+
+    def test_parse_int(self):
+        value = self.opt.parse(42)
+        self.assertEqual(value, '42')
+
+    def test_parse_bool(self):
+        value = self.opt.parse(False)
+        self.assertEqual(value, 'False')
 
     def test_default(self):
         opt = StringConfigOption()
@@ -334,7 +366,7 @@ class TestLinesConfigOption(unittest.TestCase):
 
 class TestTupleConfigOption(unittest.TestCase):
     def test_init(self):
-        opt = TupleConfigOption(2)
+        opt = TupleConfigOption(length=2)
         self.assertEqual(opt.length, 2)
 
     def test_init_no_length(self):
@@ -373,7 +405,7 @@ class TestTupleConfigOption(unittest.TestCase):
         self.assertRaises(ValueError, parser.values)
 
     def test_default(self):
-        opt = TupleConfigOption(2)
+        opt = TupleConfigOption(length=2)
         self.assertEqual(opt.default, ())
 
 
@@ -384,11 +416,11 @@ class TestDictConfigOption(unittest.TestCase):
         self.assertEqual(opt.strict, False)
 
         spec = {'a': IntConfigOption(), 'b': BoolConfigOption()}
-        opt = DictConfigOption(spec)
+        opt = DictConfigOption(spec=spec)
         self.assertEqual(opt.spec, spec)
         self.assertEqual(opt.strict, False)
 
-        opt = DictConfigOption(spec, strict=True)
+        opt = DictConfigOption(spec=spec, strict=True)
         self.assertEqual(opt.spec, spec)
         self.assertEqual(opt.strict, True)
 
@@ -414,10 +446,11 @@ baz=42
 
     def test_parse_dict(self):
         class MySchema(Schema):
-            foo = DictConfigOption({'bar': StringConfigOption(),
-                                    'baz': IntConfigOption(),
-                                    'bla': BoolConfigOption(),
-                                    })
+            foo = DictConfigOption(spec={
+                'bar': StringConfigOption(),
+                'baz': IntConfigOption(),
+                'bla': BoolConfigOption(),
+            })
         config = StringIO("""[__main__]
 foo = mydict
 [mydict]
@@ -438,10 +471,11 @@ bla=Yes
 
     def test_parse_raw(self):
         class MySchema(Schema):
-            foo = DictConfigOption({'bar': StringConfigOption(),
-                                    'baz': IntConfigOption(),
-                                    'bla': BoolConfigOption(),
-                                    })
+            foo = DictConfigOption(spec={
+                'bar': StringConfigOption(),
+                'baz': IntConfigOption(),
+                'bla': BoolConfigOption(),
+            })
         config = StringIO("""[__main__]
 foo = mydict
 [mydict]
@@ -457,7 +491,7 @@ baz=42
 
     def test_parse_invalid_key_in_parsed(self):
         class MySchema(Schema):
-            foo = DictConfigOption({'bar': IntConfigOption()})
+            foo = DictConfigOption(spec={'bar': IntConfigOption()})
 
         config = StringIO("[__main__]\nfoo=mydict\n[mydict]\nbaz=2")
         expected_values = {'__main__': {'foo': {'bar': 0, 'baz': '2'}}}
@@ -467,8 +501,9 @@ baz=42
 
     def test_parse_invalid_key_in_spec(self):
         class MySchema(Schema):
-            foo = DictConfigOption({'bar': IntConfigOption(),
-                                    'baz': IntConfigOption(fatal=True)})
+            foo = DictConfigOption(spec={
+                'bar': IntConfigOption(),
+                'baz': IntConfigOption(fatal=True)})
 
         config = StringIO("[__main__]\nfoo=mydict\n[mydict]\nbar=2")
         parser = SchemaConfigParser(MySchema())
@@ -476,12 +511,12 @@ baz=42
         self.assertRaises(ValueError, parser.parse_all)
 
     def test_default(self):
-        opt = DictConfigOption({})
+        opt = DictConfigOption(spec={})
         self.assertEqual(opt.default, {})
 
     def test_parse_no_strict_missing_args(self):
         class MySchema(Schema):
-            foo = DictConfigOption({'bar': IntConfigOption()})
+            foo = DictConfigOption(spec={'bar': IntConfigOption()})
 
         config = StringIO("[__main__]\nfoo=mydict\n[mydict]")
         expected_values = {'__main__': {'foo': {'bar': 0}}}
@@ -520,7 +555,7 @@ wham=42
     def test_parse_strict(self):
         class MySchema(Schema):
             spec = {'bar': IntConfigOption()}
-            foo = DictConfigOption(spec, strict=True)
+            foo = DictConfigOption(spec=spec, strict=True)
 
         config = StringIO("[__main__]\nfoo=mydict\n[mydict]\nbar=2")
         expected_values = {'__main__': {'foo': {'bar': 2}}}
@@ -532,7 +567,7 @@ wham=42
         class MySchema(Schema):
             spec = {'bar': IntConfigOption(),
                     'baz': IntConfigOption()}
-            foo = DictConfigOption(spec, strict=True)
+            foo = DictConfigOption(spec=spec, strict=True)
 
         config = StringIO("[__main__]\nfoo=mydict\n[mydict]\nbar=2")
         expected_values = {'__main__': {'foo': {'bar': 2, 'baz': 0}}}
@@ -543,7 +578,7 @@ wham=42
     def test_parse_strict_extra_vars(self):
         class MySchema(Schema):
             spec = {'bar': IntConfigOption()}
-            foo = DictConfigOption(spec, strict=True)
+            foo = DictConfigOption(spec=spec, strict=True)
 
         config = StringIO("[__main__]\nfoo=mydict\n[mydict]\nbar=2\nbaz=3")
         parser = SchemaConfigParser(MySchema())
@@ -554,11 +589,12 @@ wham=42
 class TestLinesOfDictConfigOption(unittest.TestCase):
     def test_parse_lines_of_dict(self):
         class MySchema(Schema):
-            foo = LinesConfigOption(
-                        DictConfigOption({'bar': StringConfigOption(),
-                                          'baz': IntConfigOption(),
-                                          'bla': BoolConfigOption(),
-                                          }))
+            foo = LinesConfigOption(item=DictConfigOption(
+                spec={
+                    'bar': StringConfigOption(),
+                    'baz': IntConfigOption(),
+                    'bla': BoolConfigOption(),
+                }))
         config = StringIO("""[__main__]
 foo = mylist0
       mylist1
@@ -590,9 +626,9 @@ class TestDictWithDicts(unittest.TestCase):
                     }
         spec = {'name': StringConfigOption(),
                 'size': IntConfigOption(),
-                'options': DictConfigOption(innerspec)}
+                'options': DictConfigOption(spec=innerspec)}
         class MySchema(Schema):
-            foo = DictConfigOption(spec)
+            foo = DictConfigOption(spec=spec)
         config = StringIO("""[__main__]
 foo = outerdict
 [outerdict]
